@@ -1,26 +1,10 @@
+package DBIx::Squirrel;
+
 use 5.010_001;
 use strict;
 use warnings;
-
-package DBIx::Squirrel;
-$DBIx::Squirrel::VERSION = '1.4.0';
-=pod
-
-=encoding UTF-8
-
-=head1 NAME
-
-DBIx::Squirrel - A C<DBI> extension
-
-=head1 VERSION
-
-version 1.4.0
-
-=cut
-
 use DBI;
 use Exporter;
-use List::Util   qw/uniq/;
 use Scalar::Util qw/reftype/;
 use Sub::Name;
 use DBIx::Squirrel::dr          ();
@@ -34,8 +18,9 @@ use namespace::clean;
 
 BEGIN {
     @DBIx::Squirrel::ISA                          = qw/DBI/;
-    *DBIx::Squirrel::EXPORT_OK                    = *DBI::EXPORT_OK;
-    *DBIx::Squirrel::EXPORT_TAGS                  = *DBI::EXPORT_TAGS;
+    $DBIx::Squirrel::VERSION                      = '1.4.1_1';
+    @DBIx::Squirrel::EXPORT_OK                    = @DBI::EXPORT_OK;
+    %DBIx::Squirrel::EXPORT_TAGS                  = %DBI::EXPORT_TAGS;
     *DBIx::Squirrel::err                          = *DBI::err;
     *DBIx::Squirrel::errstr                       = *DBI::errstr;
     *DBIx::Squirrel::rows                         = *DBI::rows;
@@ -45,13 +30,8 @@ BEGIN {
     *DBIx::Squirrel::connect_cached               = *DBIx::Squirrel::dr::connect_cached;
     *DBIx::Squirrel::FINISH_ACTIVE_BEFORE_EXECUTE = *DBIx::Squirrel::st::FINISH_ACTIVE_BEFORE_EXECUTE;
     *DBIx::Squirrel::DEFAULT_SLICE                = *DBIx::Squirrel::Iterator::DEFAULT_SLICE;
-    *DBIx::Squirrel::DEFAULT_BUFFER_SIZE          = *DBIx::Squirrel::Iterator::DEFAULT_BUFFER_SIZE;
-    *DBIx::Squirrel::BUFFER_SIZE_LIMIT            = *DBIx::Squirrel::Iterator::BUFFER_SIZE_LIMIT;
-
-    unless (defined $DBIx::Squirrel::VERSION) {
-        my $v = "1.4.0";
-        *DBIx::Squirrel::VERSION = \$v;
-    }
+    *DBIx::Squirrel::DEFAULT_CACHE_SIZE           = *DBIx::Squirrel::Iterator::DEFAULT_CACHE_SIZE;
+    *DBIx::Squirrel::CACHE_SIZE_LIMIT             = *DBIx::Squirrel::Iterator::CACHE_SIZE_LIMIT;
 }
 
 use constant E_BAD_ENT_BIND     => 'Cannot associate with an invalid object';
@@ -155,6 +135,14 @@ __END__
 
 =encoding UTF-8
 
+=head1 NAME
+
+DBIx::Squirrel - A C<DBI> extension
+
+=head1 VERSION
+
+version 1.4.1_1
+
 =head1 SYNOPSIS
 
     # ------------------
@@ -966,7 +954,6 @@ accessible via C<DBIx::Squirrel>.
     $clone_dbh = DBIx::Squirrel->connect($dbh, \%attr)
                 or die $DBIx::Squirrel::errstr;
 
-
 =head4 C<connect_cached> *
 
     $dbh = DBIx::Squirrel->connect_cached($data_source, $username, $password)
@@ -1027,7 +1014,6 @@ the statement, as well as the statement handle:
     ($rows, $sth) = $dbh->do($statement, undef, \%bind_mappings)
                 or die ...;
 
-
 =head4 C<iterate>
 
     $itor = $dbh->iterate($statement)
@@ -1071,7 +1057,6 @@ the statement, as well as the statement handle:
     $itor = $dbh->iterate($statement, undef, \%bind_mappings, @transforms)
                 or die ...;
 
-
 =head4 C<prepare> *
 
     $sth = $dbh->prepare($statement)          or die $dbh->errstr;
@@ -1088,7 +1073,6 @@ Statement-strings will be "normalised" to use the legacy C<?> style, before
 being handed-off to the C<DBI> method of the same name. In spite of this,
 you should still use key-value bindings if you opted for named placeholders.
 
-
 =head4 C<prepare_cached> *
 
     $sth = $dbh->prepare_cached($statement)
@@ -1108,7 +1092,6 @@ you should still use key-value bindings if you opted for named placeholders.
 
 It is the normalised form of the statement that is cached by the C<DBI>.
 
-
 =head4 C<results>
 
     $itor = $dbh->results($statement)
@@ -1161,7 +1144,6 @@ It is the normalised form of the statement that is cached by the C<DBI>.
     $sth->bind(%bind_mappings);
     $sth->bind(\%bind_mappings);
 
-
 =head4 C<bind_param> *
 
     $sth->bind_param($p_num, $bind_value);
@@ -1171,19 +1153,17 @@ It is the normalised form of the statement that is cached by the C<DBI>.
     $sth->bind_param($p_name, $bind_value, \%attr);
     $sth->bind_param($p_name, $bind_value, $bind_type);
 
-
 =head4 C<execute> *
 
-    $rv = $sth->execute;
+    $rv = $sth->execute();
     $rv = $sth->execute(@bind_values);
     $rv = $sth->execute(\@bind_values);
     $rv = $sth->execute(%bind_mappings);
     $rv = $sth->execute(\%bind_mappings);
 
-
 =head4 C<iterate>
 
-    $itor = $sth->iterate
+    $itor = $sth->iterate()
                 or die $dbh->errstr;
     $itor = $sth->iterate(@bind_values)
                 or die ...;
@@ -1204,10 +1184,9 @@ It is the normalised form of the statement that is cached by the C<DBI>.
     $itor = $sth->iterate(\%bind_mappings, @transforms)
                 or die ...;
 
-
 =head4 C<results>
 
-    $itor = $sth->results
+    $itor = $sth->results()
                 or die $dbh->errstr;
     $itor = $sth->results(@bind_values)
                 or die ...;
@@ -1228,12 +1207,12 @@ It is the normalised form of the statement that is cached by the C<DBI>.
     $itor = $sth->results(\%bind_mappings, @transforms)
                 or die ...;
 
-=head3 Iterator Objects
+=head3 Iterator Methods
 
 =head4 C<all>
 
-    @results = $itor->all;
-    $results_or_undef = $itor->all;
+    @results = $itor->all();
+    $results_or_undef = $itor->all();
 
 Executes the iterator's underlying statement handle object.
 
@@ -1244,11 +1223,18 @@ When called in scalar-context, this method returns a reference to
 an array of all matching row objects. Where no rows are matched,
 C<undef> would be returned.
 
-
 =head4 C<buffer_size>
 
-    $buffer_size = $itor->buffer_size;
-    $itor = $itor->buffer_size($buffer_size);
+Deprecated alias for C<cache_size>.
+
+=head4 C<buffer_size_slice>
+
+Deprecated alias for C<cache_size_slice>.
+
+=head4 C<cache_size>
+
+    $cache_size = $itor->cache_size();
+    $itor = $itor->cache_size($cache_size);
 
 May be used to determine how many results the iterator makes available to
 fetch following each trip to the database.
@@ -1266,15 +1252,16 @@ at that value, preventing the kind of automatic adjustment described above.
 
 The following package globals define the relevant default settings:
 
-    $DBIx::Squirrel::Iterator::DEFAULT_BUFFER_SIZE = 2;   # initial buffer-size
-    $DBIx::Squirrel::Iterator::BUFFER_SIZE_LIMIT   = 64;  # maximum buffer-size
+    $DBIx::Squirrel::Iterator::DEFAULT_CACHE_SIZE = 2;   # initial buffer-size
+    $DBIx::Squirrel::Iterator::CACHE_SIZE_LIMIT   = 64;  # maximum buffer-size
 
+=head4 C<cache_size_slice>
 
-=head4 C<buffer_size_slice>
-
-    ($buffer_size, $slice) = $itor->buffer_size_slice();
-    $itor = $itor->buffer_size_slice($slice, $buffer_size);
-    $itor = $itor->buffer_size_slice($buffer_size, $slice);
+    ($cache_size, $slice) = $itor->cache_size_slice();
+    $itor = $itor->cache_size_slice($slice);
+    $itor = $itor->cache_size_slice($cache_size);
+    $itor = $itor->cache_size_slice($slice, $cache_size);
+    $itor = $itor->cache_size_slice($cache_size, $slice);
 
 May be used to determine (a) how the iterator slices the results it fetches
 from the database, and (b) how many results it makes available to fetch
@@ -1285,7 +1272,7 @@ properties is returned:
 
 =over
 
-=item * C<$buffer_size>
+=item * C<$cache_size>
 
 The current size of the results buffer. That is, the current maximum number of
 results that are processed and ready to fetch after each trip to the database.
@@ -1312,13 +1299,12 @@ at that value, preventing the kind of automatic adjustment described above.
 The following package globals define the relevant default settings:
 
     $DBIx::Squirrel::Iterator::DEFAULT_SLICE       = [];  # slicing strategy
-    $DBIx::Squirrel::Iterator::DEFAULT_BUFFER_SIZE = 2;   # initial buffer-size
-    $DBIx::Squirrel::Iterator::BUFFER_SIZE_LIMIT   = 64;  # maximum buffer-size
-
+    $DBIx::Squirrel::Iterator::DEFAULT_CACHE_SIZE = 2;   # initial buffer-size
+    $DBIx::Squirrel::Iterator::CACHE_SIZE_LIMIT   = 64;  # maximum buffer-size
 
 =head4 C<count>
 
-    $count = $itor->count;
+    $count = $itor->count();
 
 Returns the total number of rows in the result set.
 
@@ -1330,10 +1316,9 @@ and the final count is returned.
 
 I<B<BEWARE> that you should not use C<next> after this method has been used!>
 
-
 =head4 C<count_fetched>
 
-    $count = $itor->count_fetched;
+    $count = $itor->count_fetched();
 
 Returns the number of results fetched so far.
 
@@ -1341,10 +1326,9 @@ If the iterator's statement has not yet been executed, it will be. Zero will
 be returned if the statement executed successfully, otherwise C<undef> is
 returned.
 
-
 =head4 C<first>
 
-    $result = $itor->first;
+    $result = $itor->first();
 
 Returns the first result in the result set, or C<undef> if there were no
 results.
@@ -1357,10 +1341,9 @@ is fetched and cached. The cached value is returned.
 
 The result of the statement's execution will be returned.
 
-
 =head4 C<iterate>
 
-    $itor_or_undef = $itor->iterate
+    $itor_or_undef = $itor->iterate()
     $itor_or_undef = $itor->iterate(@bind_values)
     $itor_or_undef = $itor->iterate(@transforms)
     $itor_or_undef = $itor->iterate(@bind_values, @transforms)
@@ -1380,10 +1363,9 @@ the iterator at the time of construction will be honoured.
 A reference to the iterator is returned if the statement was successfully
 executed, otherwise the method returns C<undef>.
 
-
 =head4 C<last>
 
-    $result = $itor->last;
+    $result = $itor->last();
 
 Returns the last result in the result set.
 
@@ -1395,10 +1377,9 @@ last result fetched is returned.
 
 I<B<BEWARE> that you should not use C<next> after this method has been used!>
 
-
 =head4 C<last_fetched>
 
-    $result = $itor->last_fetched;
+    $result = $itor->last_fetched();
 
 Returns the last result fetched.
 
@@ -1408,10 +1389,9 @@ C<undef> is returned regardless of the statement execution's outcome.
 If the statement was previously executed then the last result fetched is
 always cached. The cached value is returned.
 
-
 =head4 C<next>
 
-    $result = $itor->next;
+    $result = $itor->next();
 
 Returns the next result in the result set.
 
@@ -1435,38 +1415,40 @@ in the result set.
 
 =back
 
-
 =head4 C<one>
 
 Alias (see C<single>).
 
-
 =head4 C<remaining>
 
-    @results = $itor->remaining;
+    @results = $itor->remaining();
     $results_or_undef = $itor->remaining;
 
-
 =head4 C<reset>
 
-    $itor = $itor->reset;
+    $itor = $itor->reset();
+    $itor = $itor->reset($slice);
+    $itor = $itor->reset($cache_size);
+    $itor = $itor->reset($slice, $cache_size);
+    $itor = $itor->reset($cache_size, $slice);
 
-Executes the iterator's underlying statement handle object and resets any
-internal state.
+Executes the iterator's underlying statement handle object with the current
+bind-values, resetting any internal state.
 
-A reference to the iterator is always returned.
+This method may also be used to set the statement cache size and/or slicing
+strategy in the same call.
 
+A reference to the iterator is returned, regardless of execution outcome.
 
 =head4 C<rows>
 
-    $rows = $itor->rows;
+    $rows = $itor->rows();
 
 Returns the number of rows aftected by non-SELECT statements.
 
-
 =head4 C<single> (or C<one>)
 
-    $result = $itor->single;
+    $result = $itor->single();
 
 Returns the first result in the result set, or C<undef> if there were no
 results.
@@ -1483,10 +1465,9 @@ If the result returned is one of many buffered, a warning will be issued:
 
 The warning is a reminder to include a LIMIT 1 constraint in the statement.
 
-
 =head4 C<slice>
 
-    $slice = $itor->slice;
+    $slice = $itor->slice();
     $itor = $itor->slice($slice);
 
 May be used to determine how the iterator slices the results it fetches
@@ -1499,12 +1480,17 @@ The following package global defines the default setting:
 
     $DBIx::Squirrel::Iterator::DEFAULT_SLICE       = [];  # slicing strategy
 
-
 =head4 C<slice_buffer_size>
 
-    ($slice, $buffer_size) = $itor->slice_buffer_size;
-    $itor = $itor->slice_buffer_size($slice, $buffer_size);
-    $itor = $itor->slice_buffer_size($buffer_size, $slice);
+Deprecated alias for C<slice_cache_size>.
+
+=head4 C<slice_cache_size>
+
+    ($slice, $cache_size) = $itor->slice_cache_size();
+    $itor = $itor->slice_cache_size($slice);
+    $itor = $itor->slice_cache_size($cache_size);
+    $itor = $itor->slice_cache_size($slice, $cache_size);
+    $itor = $itor->slice_cache_size($cache_size, $slice);
 
 May be used to determine (a) how the iterator slices the results it fetches
 from the database, and (b) how many results it makes available to fetch
@@ -1520,7 +1506,7 @@ properties is returned:
 The how the iterator slices results fetched from the database. This may be an
 ARRAYREF or a HASHREF.
 
-=item * C<$buffer_size>
+=item * C<$cache_size>
 
 The current size of the results buffer. That is, the current maximum number of
 results that are processed and ready to fetch after each trip to the database.
@@ -1542,13 +1528,12 @@ at that value, preventing the kind of automatic adjustment described above.
 The following package globals define the relevant default settings:
 
     $DBIx::Squirrel::Iterator::DEFAULT_SLICE       = [];  # slicing strategy
-    $DBIx::Squirrel::Iterator::DEFAULT_BUFFER_SIZE = 2;   # initial buffer-size
-    $DBIx::Squirrel::Iterator::BUFFER_SIZE_LIMIT   = 64;  # maximum buffer-size
-
+    $DBIx::Squirrel::Iterator::DEFAULT_CACHE_SIZE = 2;   # initial buffer-size
+    $DBIx::Squirrel::Iterator::CACHE_SIZE_LIMIT   = 64;  # maximum buffer-size
 
 =head4 C<start>
 
-    $rv_or_undef = $itor->start
+    $rv_or_undef = $itor->start()
     $rv_or_undef = $itor->start(@bind_values)
     $rv_or_undef = $itor->start(@transforms)
     $rv_or_undef = $itor->start(@bind_values, @transforms)
@@ -1565,13 +1550,93 @@ resetting the iterator's internal state.
 When called with no arguments, any bind-values and transformations passed to
 the iterator at the time of construction are used.
 
-
 =head4 C<sth>
 
-    $sth = $itor->sth;
+    $sth = $itor->sth();
 
 Returns the iterator's underlying statement handle object.
 
+=head3 Iterator Exports
+
+The C<DBIx::Squirrel::Iterator> package exports a number of subroutines that
+may be used within the stages of a transformation pipeline. These provide
+information about the current transformation context.
+
+=head4 C<database>
+
+    my $dbh = database();
+
+Returns a reference to the current iterator's database handle object.
+
+=head4 C<iterator>
+
+    my $itor = iterator();
+
+Returns a reference to the current iterator.
+
+=head4 C<result>
+
+    my $result = result();
+
+Returns the result as it was when it entered the I<current stage> of the
+transformation pipeline.
+
+There are alternative ways to get at this value without having to pollute
+your namespace, although C<result> is less cryptic:
+
+=over
+
+=item *
+
+It is the first element of the C<@_> list. Use C<$_[0]>, or C<shift> the
+value from the start of the list.
+
+=item *
+
+It is also presented in the ephemeral C<$_> variable. It should be consumed
+quickly, before it changes.
+
+=back
+
+=head4 C<result_current>
+
+An alias (see C<result>).
+
+=head4 C<result_first>
+
+    my $result = result_first();
+
+Returns the first result fetched.
+
+=head4 C<result_prev>
+
+    my $result = result_previous();
+
+Returns the previous result, if there was one.
+
+=head4 C<result_previous>
+
+An alias (see C<result_prev>).
+
+=head4 C<result_offset>
+
+    my $offset = result_offset();
+
+Returns the result's zero-based offset, effectively the number of results
+fetched, less one.
+
+=head4 C<result_original>
+
+    my $result = result_original();
+
+Returns the result as it was when it entered the I<first stage> of the current
+transformation pipeline.
+
+=head4 C<statement>
+
+    my $sth = statement();
+
+Returns a reference to the current iterator's statement handle object.
 
 =head1 COPYRIGHT AND LICENSE
 
